pro fluxtube2

;................................................................
; create test model for testing R-K 4th order integration
; second order thin FT approximation
; see Pneuman et al 1986
;................................................................

       common der,dx,dz,dxdz_tot
       common atmosfera,x,z,t0,p0,rho0,bx,bz

; original
nx=216.
nz=251.
a = 100       ;tube radius
alfa=0.05     ;filling factor
beta_base=4. ;beta at Z=0 tube axis
coef_be=0.001   ;coef_be=Bes/bs
coef_p =0.01    ;coef. for 2nd order pressure term at z=0
coef_hb=1.005 ;0.99

small=0.1 ;alfa<0.2

epsilon=1.e-10
error=10.
maxiter=10000

;................................................................
BK=1.380662E-16 & RGAS=8.3143e7 & G=2.74e+4
MH=1.67339E-24 & gamma=5./3.
;................................................................

Bx=dblarr(nx,nz) & Bz=Bx
P0=Bx & Rho0=Bx & T0=Bx

dx=8. ;km, step in X
dz=8. ;km, step in Z

z=dindgen(nz)*dz
x=dindgen(nx)*dx

zzero=0
m=min(abs(z),zzero) & print,'z equal to zero at ',zzero

print,'max height ',max(z)

;................................................................
; initial conditions
; need P_ext at z=0, P_int at z=0, r=0, Bs at z=0, r=0
;      T_int(z,x), T_ext(z)
;................................................................
phot,Z_PH,T_PH,RHO_PH,PG_PH,PE_PH,HP_PH,MU_PH

;................................................................
;calculate external pressure and temperature
pext=10^interpol(alog10(pg_ph),z_ph,z)
text=interpol(t_ph,z_ph,z)
rext=10^interpol(alog10(rho_ph),z_ph,z)
hext=interpol(HP_PH,z_ph,z)
mext=interpol(MU_PH,z_ph,z)

pext1=pext

;recalculate pressure according to scale height
for iz=1,nz-1 do pext(iz)=pext(0)*exp(int_tabulated(z[0:iz],-1./hext[0:iz]))
pext=pext/pext[zzero]*pext1[zzero]

mext=rext*8.3143e7*text/pext

Bs=sqrt(8*!PI*gamma*pext[zzero]/(2*beta_base + gamma))
print,'field strength at the photosphere ',Bs,' G'
Bes=coef_be*Bs



;................................................................
;initial conditions

taxis=text    ;temperature inside the tube is the same, no X dependence
haxis=hext    ;--> scale height is the same

hs=hext[0] ;[zzero]
as=a/hs       ;dimensionless tube radius

HB=2*haxis*coef_hb
baxis=dblarr(nz)+Bs
for iz=1,nz-1 do baxis[iz]=Bs*exp(int_tabulated(z[0:iz],-1./HB[0:iz]))
baxis=baxis/baxis[zzero]*Bs

;normalize at the value at the deepest layer
Bs=baxis[0]
Bes=coef_be*Bs
;....new!!!!!!!

h0=dblarr(nz)
dh0dy=dblarr(nz)  ;dh0/dy

h0[0]=Bs/Bs                               ;initial value for h0=1.
dh0dy[0]=-baxis/HB*hs/bs  ;initial value for dh0/dy, as for the thin flux tube

pes=pext[0]                         ;initial value for extrernall pressure
ps=pes - Bs*Bs/8./!pi + Bes*Bes/8./!pi   ;initial value for internal pressure, as for the thin fluxtube
if ps lt 0. then stop,'negative pressure'


paxis=dblarr(nz)+ps
for iz=1,nz-1 do paxis(iz)=ps*exp(int_tabulated(z[0:iz],-1./haxis[0:iz]))

QU=(paxis - pext)*16.*!pi/as^2/Bs^2  ;pressure deficit, negative

y=z/hs                               ;normalized distance
dy=dz/hs                             ;normalized step

coef_b=2.*coef_be^2/as^2

beta=4*!pi*ps/bs^2
;................................................................

;................................................................
;Runge-Kutta
; initial approximation for the first 3-4 points
dy1=dy/10.
nz1=40.
y1=dindgen(nz1)*dy1
QU1=interpol(QU,y,y1)  ;interpolate Q-term to finer grid
h0_first=dblarr(nz1) & h0_first[0]=h0[0]
dh0dy_first=dblarr(nz1) & dh0dy_first[0]=dh0dy[0]

FOR I=1,NZ1-1 DO BEGIN

BTERM11=coef_b*h0_first[i-1]^2*(1. - alfa)^2 / (h0_first[i-1] - alfa)^2

k1=( dh0dy_first[i-1]^2/2./h0_first[i-1] + 2.*h0_first[i-1]^2/as^2 + QU1[i-1] - BTERM11 )*dy1

QU12=interpol(QU1,y1,y1[i-1]+dy1/2.)
h12 = h0_first[i-1] + dh0dy_first[i-1]*dy1/2.
dh12 = dh0dy_first[i-1] + k1/2.
BTERM12=coef_b*h12^2*(1. - alfa)^2 / (h12 - alfa)^2

k2 =( dh12^2/2./h12 + 2.*h12^2/as^2 + QU12 - BTERM12 )*dy1

QU13=QU12
h13 = h0_first[i-1] + dh0dy_first[i-1]*dy1/2. + k1/4.*dy1
dh13 = dh0dy_first[i-1] + k2/2.
BTERM13=coef_b*h13^2*(1. - alfa)^2 / (h13 - alfa)^2

k3 =( dh13^2/2./h13 + 2.*h13^2/as^2 + QU13 - BTERM13 )*dy1

QU14=QU1[i]
h14 = h0_first[i-1] + dh0dy_first[i-1]*dy1 + k2/2.*dy1
dh14 = dh0dy_first[i-1] + k3
BTERM14=coef_b*h14^2*(1. - alfa)^2 / (h14 - alfa)^2

k4 =( dh14^2/2./h14 + 2.*h14^2/as^2 + QU14 - BTERM14 )*dy1

h0_first[i] = h0_first[i-1] + dh0dy_first[i-1]*dy1 + (k1 + k2 + k3)*dy1/6.

dh0dy_first[i] = dh0dy_first[i-1] + (k1 + 2.*k2 + 2.*k3 + k4)/6.

ENDFOR


;................................................................
;given first 3-4 points interate the rest (Korn, p.708)
h0[1]=h0_first[10]
h0[2]=h0_first[20]
h0[3]=h0_first[30]

dh0dy[1]=dh0dy_first[10]
dh0dy[2]=dh0dy_first[20]
dh0dy[3]=dh0dy_first[30]

h0[4:*]=baxis[4:*]/Bs                               ;initial value for h0=1.
dh0dy[4:nz-2]=(baxis[5:nz-1] - baxis[4:nz-2])/dz/Bs*hs   ;initial value for dh0/dy, as for the thin flux tube
dh0dy[nz-1]=dh0dy[nz-2]

funct=dblarr(nz)

for i=0,nz-1 do begin
BTERM=coef_b*h0[i]^2*(1. - alfa)^2 / (h0[i] - alfa)^2
funct[i]=dh0dy[i]^2/2./h0[i] + 2.*h0[i]^2/as^2 + QU[i] - BTERM
endfor


window,0
!p.multi=[0,2,2]
plot,y*hs,baxis/bs,title='h0'

;have 4 initial points, start interations
FOR i=3, NZ-2 DO BEGIN
iter=0

while(error gt epsilon and iter le maxiter) do begin
iter=iter+1

h0[i]=2.*h0[i-1] - h0[i-2] + dy*dy/12.*(10.*funct[i-1] + funct[i] + funct[i-2])

dh0dy[i]=dh0dy[i-2] + dy/3.*(4.*funct[i-1] + funct[i] + funct[i-2])

  if( (h0[i] - alfa) lt small or h0[i] gt h0[i-1]) then begin

     h0[i]=alfa+10^interpol(alog10(h0[0:i-1]-alfa),y[0:i-1],y[i])
     dh0dy[i]=(h0[i]-h0[i-1])/dy
  endif


BTERM=coef_b*h0[i]^2*(1. - alfa)^2 / (h0[i] - alfa)^2
funct_new=dh0dy[i]^2/2./h0[i] + 2.*h0[i]^2/as^2 + QU[i] - BTERM

error=abs(funct_new-funct[i])/abs(funct_new)
;print,i,iter,error,funct_new,funct[i]
funct[i]=funct_new
funct[i+1]=funct[i-3]

endwhile

error=10.

oplot,y[0:i]*hs,h0[0:i],line=2

ENDFOR

;boundary conditions

h0[NZ-1]=h0[NZ-2]
dh0dy[NZ-1]=dh0dy[NZ-2]

plot,y*hs,dh0dy,title='dh0hy',psym=3,yrange=[-1.,1.]

b=bs*h0
diff=pext-b*b/8./!pi+bes*bes/8./!pi
beta0=paxis/b^2*8*!pi

;................................................................
;calculate the rest of variables

f1=fltarr(nz)
f1=-0.5*dh0dy

p2=fltarr(nz)
p2(*)=h0*paxis*coef_p   ;coef_p=ps2/ps

h2=fltarr(nz)
hb1=fltarr(nz)
HB1[1:*]=-z[1:*]/alog(h0[1:*]) & hb1[0]=hb1[1]
h_sec=h0*(deriv(z,hb1)+1)/hb1^2*hs*hs & h_sec[0]=h_sec[1]
h2(*)=-0.25*h_sec - beta*p2/ps


oplot,y*hs,h_sec,line=2

plot,y*hs,alog10(beta0),title='log(beta)'

avar=as/sqrt(h0)*hs
bprof=fltarr(nx,nz)
rc=nx/2

for iz=0,nz-1 do begin
r0=round(avar[iz]/dx)
if r0 ge nx/2 then bprof[*,iz]=1. else bprof[rc-r0:rc+r0,iz]=1.
endfor

tvframe,bprof
bprof_z=bprof


for iz=0,nz-1 do begin
r0=round(avar[iz]/dx)
  if r0 ge nx/2 then bprof[*,iz]=x-x[rc] else bprof[rc-r0:rc+r0,iz]=x[rc-r0:rc+r0]-x[rc]
endfor

bprof_x=bprof


for i=0,nx-1 do begin ;if stratified
Bz[i,*]=(h0 + (x[i]-x[rc])^2*h2/hs/hs)*bprof_z[i,*]*bs + bes*(1.-bprof_z[i,*])
Bx[i,*]=bprof_x[i,*]*f1*bs/hs
P0[i,*]=pext*(1.-bprof_z[i,*]) + (paxis + (x[i]-x[rc])^2*p2/hs/hs)*bprof_z[i,*]
T0[i,*]=taxis
Rho0[i,*]=P0[i,*]/RGAS/T0[i,*]*MEXT
endfor



dd=fltarr(nx,nz)
for i=0,nz-1 do dd(0:nx-2,i)=(bprof_z(1:nx-1,i) - bprof_z(0:nx-2,i))

m=max(dd(*,nz-1),ix1) & ix1=ix1+1+3  ;previous models
m=min(dd(*,nz-1),ix2) & ix2=ix2-3

ntube=ix2-ix1+1
nrest=nx-ntube

for i=0,nz-1 do begin
Bz[ix1:ix2,i]=Bz[ix1:ix2,i]
Bz[0:ix1-1,i]=Bz[ix2-ix1:ix2-1,i]
Bz[ix2+1:*,i]=Bz[ix1+1:nx-ix2+ix1-1,i]

Bx[ix1:ix2,i]=Bx[ix1:ix2,i]
Bx[0:ix1-1,i]=Bx[ix2-ix1:ix2-1,i]        ;-
Bx[ix2+1:*,i]=Bx[ix1+1:nx-ix2+ix1-1,i]   ;-

P0[ix1:ix2,i]=P0[ix1:ix2,i]
P0[0:ix1-1,i]=P0[ix2-ix1:ix2-1,i]
P0[ix2+1:*,i]=P0[ix1+1:nx-ix2+ix1-1,i]

T0[ix1:ix2,i]=T0[ix1:ix2,i]
T0[0:ix1-1,i]=T0[ix2-ix1:ix2-1,i]
T0[ix2+1:*,i]=T0[ix1+1:nx-ix2+ix1-1,i]

Rho0[ix1:ix2,i]=Rho0[ix1:ix2,i]
Rho0[0:ix1-1,i]=Rho0[ix2-ix1:ix2-1,i]
Rho0[ix2+1:*,i]=Rho0[ix1+1:nx-ix2+ix1-1,i]

endfor


window,1
tvframe,alog10(bz),/bar,title='alog(Bz)'
tvframe,bx,/bar,title='Bx'
tvframe,alog10(p0),/bar,title='alog(P0)'
tvframe,alog10(rho0),/bar,title='alog(Rho0)'

print,'min bz ',min(bz)


dbz=fltarr(nx,nz)
for ix=0,nx-1 do dbz(ix,*)=deriv(z,bz(ix,*))
dbx=fltarr(nx,nz)
for iz=0,nz-1 do dbx(*,iz)=deriv(x,(x-x[rc])*bx(*,iz))
for iz=0,nz-1 do dbx[0:rc-1,iz]=dbx[0:rc-1,iz]/(x[0:rc-1]-x[rc])
for iz=0,nz-1 do dbx[rc+1:*,iz]=dbx[rc+1:*,iz]/(x[rc+1:*]-x[rc])
dbx[rc,*]=0.5*(dbx[rc-1,*]+dbx[rc+1,*])
div=dbx+dbz
pdif=p0+bz^2/8./!pi + bx^2/8./!Pi

window,2
tvframe,-5>div<5,/bar,title='div(b)'
tvframe,alog10(pdif),/bar,title='P dif'

dxdz_tot=bx/bz
xx0=x[nx/2-round(a/dx)]+ 2.*a*findgen(11)/10.

runge,xx0(0),xout,z & plot,xout,z,xrange=[0,x[nx-1]],yrange=[0,z[nz-1]]
for i=0,n_elements(xx0)-1 do begin & runge,xx0(i),xout,z & oplot,xout,z & endfor

       cs=sqrt(gamma*P0/Rho0)               ; cm s-1
       b0=sqrt(bx^2 + bz^2)          ; G
       va=b0/sqrt(4.*!PI*rho0)        ; cm s-1
       beta00 = cs*cs/va/va

contour,beta00,x,z,levels=[1.],thick=2,title='beta contours',/follow
contour,beta00,x,z,levels=[0.1],/overplot,thick=1,/follow
contour,beta00,x,z,levels=[10.],/overplot,thick=3,/follow

print,'min beta',min(beta00)
print,'max va',max(va)*1.e-5



outname='fluxtube2.dat'
openw,10,outname
print,outname
printf,10,nx,nz
print,nx,nz
printf,10,'horizontal x'
printf,10,x
printf,10,'vertical'
printf,10,z
printf,10,'temperature'
printf,10,t0
printf,10,'pressure'
printf,10,p0
printf,10,'density'
printf,10,rho0
printf,10,'Bx'
printf,10,bx
printf,10,'Bz'
printf,10,bz
printf,10,'a,alfa,beta_base,coef_be,coef_p,coef_hb,small'
printf,10,a,alfa,beta_base,coef_be,coef_p,coef_hb,small

close,10

end
